package net.sourceforge.rtf.usecases.annotation;

import java.io.File;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import net.sourceforge.rtf.template.IContext;
import net.sourceforge.rtf.usecases.AbstractRTFUseCase;
import net.sourceforge.rtf.usecases.model.Dependency;
import net.sourceforge.rtf.usecases.model.Developer;
import net.sourceforge.rtf.usecases.model.Project;
import net.sourceforge.rtf.usecases.util.UseCaseLog;
import net.sourceforge.rtf.usecases.util.UseCaseUtil;

public class RTFAnnotationWithVelocity extends AbstractRTFUseCase {

    public RTFAnnotationWithVelocity(String outDirectory) {
        super(outDirectory);
    }

    protected void putContext(IContext context) {
        /*
         * Context of simply POJO
         */
        context.put("date", new Date());
        context.put("project", new Project("Jakarta Velocity"));

        context.put("header_developer_name", "Name");
        context.put("header_developer_email", "Email");
        context.put("header_developer_roles", "Roles");
        /*
         * Context of list of POJO (Developers and Dependencies)
         */
        List developers = new ArrayList();
        Developer developer = new Developer("Will Glass-Husain",
                "wglass@apache.org");
        developer.addRole("Java Developer");
        developer.addRole("Release Manager");
        developers.add(developer);
        developer = new Developer("Geir Magnusson Jr.", "geirm@apache.org");
        developer.addRole("Java Developer");
        developer.addRole("Release Manager");
        developers.add(developer);
        developer = new Developer("Daniel L. Rall", "dlr@finemaltcoding.comg");
        developer.addRole("Java Developer");
        developer.addRole("Release Manager");
        developers.add(developer);
        context.put("developers", developers);

        List dependencies = new ArrayList();
        Dependency dependency = new Dependency("commons-collection", "jar",
                "1.0", "http://jakarta.apache.org/commons/collection/");
        dependencies.add(dependency);
        dependency = new Dependency("logkit", "jar", "1.0.1", "http://logkit");
        dependencies.add(dependency);
        dependency = new Dependency("oro", "jar", "2.0.8",
                "http://jakarta.apache.org/oro/");
        dependencies.add(dependency);

        context.put("dependencies", dependencies);

    }

    /**
     * Use case of RTFTemplate with the RTF source model
     * <b>jakarta-velocity-model.rtf</b>. It generate three files into
     * directory <b>usecases/out/jakartavelocityproject</b> :
     * <ul>
     * <li><b>jakarta-velocity-model.rtf.&lt;rtfTemplateImpl&gt;.rtf</b> RTF
     * document transformed.</li>
     * </ul>
     * <ul>
     * <li><b>jakarta-velocity-model.rtf.out.rtf</b> RTF final with value of
     * the context.</li>
     * </ul>
     * <ul>
     * <li><b>jakarta-velocity.fields.xml</b> XML fields available.</li>
     * </ul>
     * 
     * @param args the args.
     * @throws Exception the exception.
     */
    public static void main(String[] args) throws Exception {
        String baseDirectory = UseCaseUtil.getBaseDirectory(args);
        UseCaseLog logger = new UseCaseLog();
        logger.logTitle("Start RTFJakartaVelocityProject...");
        String outDirectory = baseDirectory
                + "usecases/out/annotation";
        // Create out Directory
        File out = new File(outDirectory);
        out.mkdir();

        // Generate RTF file by using RTF model jakarta-velocity-model.rtf
        String rtfSource = baseDirectory
                + "usecases/models/annotation/annotation.rtf";
        RTFAnnotationWithVelocity usecase = new RTFAnnotationWithVelocity(
                outDirectory);
        /**
         * Set Velocity as RTFTemplate implementation
         * 
         */
        // usecase.setRtfTemplateImpl(RTFTemplateHelper.DEFAULT_VELOCITY_RTFTEMPLATE);
        // => this line is
        // not required, because by default velocity is the default RTFTemplate
        // implementation
        usecase.saveTransformedDocument(true); // Save RTF file with velocity
                                                // macro
        usecase.run(rtfSource);

        logger.logTitle("GENERATE RTF VELOCITY AND TARGET FILES INTO : "
                + outDirectory);

        // Save XML fields into file with update
        String xmlFieldsAvailable = outDirectory
                + "/jakartavelocityproject.fields.xml";
        usecase.saveXmlFields(xmlFieldsAvailable, false);
        logger.logTitle("GENERATE XML FIELDS AVAIALABLE : "
                + xmlFieldsAvailable);
        logger.logBody(usecase.getXMLFields());

        logger.logTitle("End RTFJakartaVelocityProject...");

    }

}
