package net.sourceforge.rtf.usecases.table;

import java.io.File;
import java.text.DateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import net.sourceforge.rtf.RTFTemplate;
import net.sourceforge.rtf.template.IContext;
import net.sourceforge.rtf.usecases.AbstractRTFUseCase;
import net.sourceforge.rtf.usecases.model.Dependency;
import net.sourceforge.rtf.usecases.model.Developer;
import net.sourceforge.rtf.usecases.model.Project;
import net.sourceforge.rtf.usecases.util.UseCaseLog;
import net.sourceforge.rtf.usecases.util.UseCaseUtil;

/**
 * Use case which RTF Table. It generate RTF file with information about Jakarta
 * Velocity Project. This example use RTF model
 * <b>usecases/models/table/table.rtf</b> RTF files generated into directory
 * <b>usecases/out/table</b>.
 * 
 * @version 1.0.0
 * @author <a href="mailto:angelo.zerr@gmail.com">Angelo ZERR</a>
 * 
 */
public class RTFTemplateWithRTFTable extends AbstractRTFUseCase {

    public RTFTemplateWithRTFTable(String outDirectory) {
        super(outDirectory);
    }

    protected void putDefaultFormat(RTFTemplate rtfTemplate) {
        /*
         * Set any common formatting objects on the engine - these will be
         * passed through to each template it creates
         */
        rtfTemplate.setDefaultFormat(Date.class, DateFormat.getDateInstance());
    }

    protected void putContext(IContext context) {
        /*
         * Context of simply POJO
         */
        context.put("date", new Date());
        context.put("project", new Project("Jakarta Velocity"));

        context.put("header_developer_name", "Name");
        context.put("header_developer_email", "Email");
        context.put("header_developer_roles", "Roles");
        /*
         * Context of list of POJO (Developers and Dependencies)
         */
        List developers = new ArrayList();
        Developer developer = new Developer("Will Glass-Husain",
                "wglass@apache.org");
        developer.addRole("Java Developer");
        developer.addRole("Release Manager");
        developers.add(developer);
        developer = new Developer("Geir Magnusson Jr.", "geirm@apache.org");
        developer.addRole("Java Developer");
        developer.addRole("Release Manager");
        developers.add(developer);
        developer = new Developer("Daniel L. Rall", "dlr@finemaltcoding.comg");
        developer.addRole("Java Developer");
        developer.addRole("Release Manager");
        developers.add(developer);
        context.put("developers", developers);

        List dependencies = new ArrayList();
        Dependency dependency = new Dependency("commons-collection", "jar",
                "1.0", "http://jakarta.apache.org/commons/collection/");
        dependencies.add(dependency);
        dependency = new Dependency("logkit", "jar", "1.0.1", "http://logkit");
        dependencies.add(dependency);
        dependency = new Dependency("oro", "jar", "2.0.8",
                "http://jakarta.apache.org/oro/");
        dependencies.add(dependency);

        context.put("dependencies", dependencies);

    }

    public static void main(String[] args) throws Exception {
        String baseDirectory = UseCaseUtil.getBaseDirectory(args);
        UseCaseLog logger = new UseCaseLog();
        logger.logTitle("Start RTFTemplateWithRTFTable...");
        String outDirectory = baseDirectory + "usecases/out/table";
        // Create out Directory
        File out = new File(outDirectory);
        out.mkdir();

        // Generate RTF file by using RTF model table.rtf
        String rtfSource = baseDirectory + "usecases/models/table/table.rtf";
        RTFTemplateWithRTFTable usecase = new RTFTemplateWithRTFTable(
                outDirectory);
        /**
         * Set Velocity as RTFTemplate implementation
         * 
         */
        // usecase.setRtfTemplateImpl(RTFTemplateHelper.DEFAULT_VELOCITY_RTFTEMPLATE);
        // => this line is
        // not required, because by default velocity is the default RTFTemplate
        // implementation
        usecase.saveTransformedDocument(true); // Save RTF file with velocity
                                                // macro
        usecase.run(rtfSource);

        logger.logTitle("GENERATE RTF VELOCITY AND TARGET FILES INTO : "
                + outDirectory);

        // Save XML fields into file with update
        String xmlFieldsAvailable = outDirectory + "/table.fields.xml";
        usecase.saveXmlFields(xmlFieldsAvailable, false);
        logger.logTitle("GENERATE XML FIELDS AVAIALABLE : "
                + xmlFieldsAvailable);
        logger.logBody(usecase.getXMLFields());

        logger.logTitle("End RTFTemplateWithRTFTable...");

    }

}
